@@ -103,22 +103,17 @@
             stop("The derivative of the constraints at theta0 is either infinite or NAN")
         if (qr(dR)$rank < length(R))
             stop("The matrix of derivatives of the constraints is not full rank")
-        rhs <- as.character(object@fRHS)
-        if (!is.null(object@fLHS))
-            lhs <- as.character(object@fLHS)
-        else
-            lhs <- NULL
+        rhs <- object@fRHS
+        lhs <- object@fLHS
+        env <-  new.env()        
         for (r in R)
-            {
-                rhs <- gsub(as.character(r[2]), paste("(", as.character(r[3]),
-                                                      ")", sep=""), rhs)
-                if (!is.null(lhs))
-                    lhs <- gsub(as.character(r[2]),
-                                paste("(", as.character(r[3]),
-                                      ")", sep=""), lhs)
-            }
-        rhs <- parse(text=rhs)
-        lhs <- parse(text=lhs)
+        {
+            assign(as.character(r[2]),
+                   parse(text=paste("(", as.character(r[3]), ")", sep=""))[[1]], env)
+        }
+        rhs <- as.expression(do.call('substitute', list(rhs[[1]], env)))
+        if (!is.null(lhs))
+            lhs <- as.expression(do.call('substitute', list(lhs[[1]], env)))
         k <- object@k-length(R)
         parNames <- object@parNames[!(object@parNames %in% rest)]
         theta0 <- object@theta0[!(object@parNames %in% rest)]        
@@ -162,27 +157,20 @@
             stop("The derivative of the constraints at theta0 is either infinite or NAN")
         if (qr(dR)$rank < length(R))
             stop("The matrix of derivatives of the constraints is not full rank")
-        rhs <- list()
-        lhs <- list()
-        for (i in 1:length(object@fRHS))
-            {
-                rhs[[i]] <- as.character(object@fRHS[[i]])
-                if (!is.null(object@fLHS[[i]]))
-                    lhs[[i]] <- as.character(object@fLHS[[i]])
-                else
-                    lhs[[i]] <- NULL      
-                for (r in R)
-                {
-                    rhs[[i]] <- gsub(as.character(r[2]), paste("(", as.character(r[3]),
-                                                               ")", sep=""), rhs[[i]])
-                    if (!is.null(lhs[[i]]))
-                        lhs[[i]] <- gsub(as.character(r[2]),
-                                         paste("(", as.character(r[3]),
-                                               ")", sep=""), lhs[[i]])
-                }
-                rhs[[i]] <- parse(text=rhs[[i]])
-                lhs[[i]] <- parse(text=lhs[[i]])
-            }
+        rhs <- object@fRHS
+        lhs <- object@fLHS
+        env <-  new.env()
+        for (r in R)
+        {
+            assign(as.character(r[2]),
+                   parse(text=paste("(", as.character(r[3]), ")", sep=""))[[1]], env)
+        }
+        for (i in 1:length(rhs))
+        {
+            rhs[[i]] <- as.expression(do.call('substitute', list(rhs[[i]][[1]], env)))
+            if (!is.null(lhs[[i]]))
+                lhs[[i]] <- as.expression(do.call('substitute', list(lhs[[i]][[1]], env)))
+        }
         k <- object@k-length(R)
         parNames <- object@parNames[!(object@parNames %in% rest)]
         if (length(parNames)!=k)
@@ -358,7 +346,7 @@ setMethod("evalDMoment", signature("rnonlinearModel"),
     n <- length(hypothesis)
     k <- length(cnames)
     # an attempt to rename all special variable names (from transformed I() e.g. or
-    # interection :. 
+    # interaction :. 
     newN <- paste("theta", 1:k, sep="")
     tmp <- cnames
     hasI <- grepl("I(", cnames, fixed=TRUE)
@@ -613,6 +601,7 @@ setGeneric("getRestrict", function(object, ...)
 
 setMethod("getRestrict", "rlinearModel",
           function(object, theta) {
+              theta <- setCoef(as(object, "linearModel"), theta)
               R <- c(object@cstLHS%*%theta)
               cst <- .printHypothesis(object@cstLHS, object@cstRHS, object@parNames)
               list(dR=object@cstLHS, R=R, q=object@cstRHS, hypo=cst,
@@ -623,6 +612,7 @@ setMethod("getRestrict", "rnonlinearModel",
           function(object, theta) {
               dR <-numeric()
               R <- numeric()
+              theta <- setCoef(as(object, "nonlinearModel"), theta)              
               for (r in object@R)
                   {
                       dlhs <- sapply(object@parNames, function(pn)
@@ -685,18 +675,7 @@ setMethod("coef", "rnonlinearModel",
           function(object, theta)
           {
               spec <- modelDims(object)
-              if (length(theta)>0)
-              {
-                  if (is.null(names(theta)))
-                  {
-                      if (length(theta)!=length(spec$parNames))
-                          stop("Wrong number of coefficients")
-                      names(theta) <- spec$parNames
-                  } else {
-                      if (!all(names(theta)%in%spec$parNames))
-                          stop("theta has wrong names")
-                  }
-              }
+              theta <- setCoef(object, theta)
               theta2 <- rep(0,object@k)
               names(theta2) <- object@parNames
               theta2[names(theta)] <- theta
